@@ -105,10 +105,10 @@ CONTAINER_K = {
   "INPUT_CHAIN": "",
   "PORTBASE": 5000,  # VNC port = VNC display number + PORTBASE
   "NETWORK_GROUP": 'network',
-  "MATLAB_GROUP": "changeme",	# example, change or override this in maxwell.conf
-  "SSH_GROUP": "changeme",	# example, change or override this in maxwell.conf
+  "MATLAB_GROUP": "changeme",	# example, change or override this in mw2-client.conf
+  "SSH_GROUP": "changeme",	# example, change or override this in mw2-client.conf
   "MYSQL_GROUP": "hubmysql",
-  "MATLAB_LICENSE_SERVER": "localhost", # example, change or override this in maxwell.conf
+  "MATLAB_LICENSE_SERVER": "localhost", # example, change or override this in mw2-client.conf
   "XAUTH_RETRIES": 10,
   "INTERNAL_PATH": '/usr/bin/', # path to mergeauth and startxvnc inside the container
   # REVISIT: code doesn't work with non zero VZOFFSET.  We should set it to 100 because 0-100 are reserved by OpenVZ.
@@ -118,7 +118,7 @@ CONTAINER_K = {
   "WIN_HOME_DIR_SERVER": "localhost", # Server where home directories are located
   "WIN_HOME_DRIVE_MAP_PATH_A": "e:\\vz\private\\", # Location for the "net use" command file (first part)
   "WIN_HOME_DRIVE_MAP_PATH_B": "\\root\\CTID_command\\map_home_dir.cmd", # Location for the "net use" command file (second part)
-  "WIN_SMB_SECRET": "--------------", # override in /etc/mw-service/maxwell-service.conf
+  "WIN_SMB_SECRET": "--------------", # override in /etc/mw-service/exec.conf
   "STUNNEL_PORTS": 4000, # stunnel ports = VNC display number + STUNNEL_PORTS
   "FILEXFER_PORTS": 9000, # file transfer ports = VNC display number + FILEXFER_PORTS
   "VZ_CONF_PATH": '/etc/vz/conf',
@@ -143,7 +143,7 @@ VIRTUALSSH_K = {
 
 MW_USER = "www-data"
 WEB_HOMEDIR = "/var/www"
-CONFIG_FILE = '/etc/mw-client/mw-client.conf'
+CONFIG_FILE = '/etc/mw-client/mw2-client.conf'
 EXEC_CONFIG_FILE = '/etc/mw-service/exec.conf'
 FILE_CONFIG_FILE = '/etc/mw-service/file.conf'
 RUN_DIR = '/usr/lib/mw'
