@@ -256,7 +256,23 @@ class Container():
       params_path = rdir + "/parameters.hz"
       pfile = open(params_path, "w")
       pfile.write(urllib2.unquote(params).decode("utf8"))
-
+      
+  def mount_paths(self, mountlist):
+    """Given a list of paths, mount them in container"""
+    for mount_pt in mountlist:
+      log("mounting %s" % (mount_pt))
+      # mount_pt must already exist
+      if not os.path.exists(mount_pt):
+        log("trying to mount '%s' but it doesn't exist" % mount_pt)
+        continue
+      if not os.path.exists(self.vz_root_path + mount_pt):
+        args = ['/bin/mkdir', '-m', '0700', '-p', self.vz_root_path + mount_pt]
+        p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        p.communicate()
+        if p.returncode != 0:
+          raise MaxwellError("Could not create '%s'" % (self.vz_root_path + mount_pt))
+      self.__root_mount(mount_pt, 'rw,noatime')
+    
   def __child_unix_command(self, user, session_id, timeout, command, params, sesslog_path):
     """child
       1. Setup the environment inside the container: permissions, password, group files,
@@ -266,7 +282,6 @@ class Container():
       5. Calculate time stats
       6. Restore the firewall rules
     """
-
     err = open(sesslog_path + ".err", 'a', 0)
     out = open(sesslog_path + ".out", 'a', 0)
     err.write("Starting command '%s' for '%s' with timeout '%s'\n" % (command, user, timeout))
@@ -280,18 +295,17 @@ class Container():
       else:
         raise MaxwellError("Anonymous user session not supported")
     else:
-      if self.k["ANONYMOUS"]:
-        # mount script already mounted this if we don't support anonymous sessions
-        self.__root_mount("/home", "rw,nodev,nosuid,noatime")
-      # Setup the environment inside the container: permissions, password, group files
-      err.write("Setup the environment inside the container: permissions, password, group files\n")
       account = User_account(user)
-      err.write("got user account info\n")
+      # err.write("got user account info\n")
+      # mount script doesn't mount /home anymore
+      self.__root_mount(account.homedir, "rw,nodev,nosuid,noatime")
+      # Setup the environment inside the container: permissions, password, group files
+      # err.write("Setup the environment inside the container: permissions, password, group files\n")
 
       # write group entries
       # Get a list of the supplementary groups...
       groups = account.groups()
-      err.write("got groups\n")
+      # err.write("got groups\n")
       try:
         self.__etc_passwd(user, account, groups, err)
       except StandardError, exc:
@@ -313,26 +327,33 @@ class Container():
       p.communicate()
 
     # 1b. /apps bind mount conditional on apps group membership
-    # NOTE: the mount script in /etc/vz/conf takes care of mounting apps when
-    # APPS_READONLY is false!  Don't try to mount it here again, it will fail
     # This requires xvnc to not be in /apps, but in /usr/sbin
-    if self.k["APPS_READONLY"]:
-      err.write("apps readonly\n")
-      if 'apps' in groups:
-        mount_opt = 'rw,acl,noatime'
-      else:
-        mount_opt = 'ro,acl,noatime'
-        log("mounting apps READONLY")
+    if 'apps' in groups:
+      err.write("%s is in apps group\n" % user)
+      mount_opt = 'rw,acl,noatime'
       self.__root_mount("/apps", mount_opt)
+      # mount home directory of user 'apps' to support su to apps
+      apps_account = User_account('apps')
+      self.__root_mount(apps_account.homedir, "rw,nodev,nosuid,noatime")
+    else:
+      err.write("%s is NOT in apps group, figuring out which directories to mount\n" % user)
+      self.__mount_subdirs_ro(err, user, "/apps", 'ro,acl,noatime')
 
-    # User-based mounts
-    # uses bind mounts from an already mounted filesystem
+    # mount /data subdirs
+    if os.path.exists('/data'):
+      for s in os.listdir('/data'):
+        self.__mount_subdirs(err, user, os.path.join('/data',s))
+      
+    # Username-based mounts
+    # uses bind mounts from an already mounted filesystem, which has a directory for each user
+    # the mounted directory is the username
     if self.k["USER_MOUNT"]:
       for mount_pt in self.k["USER_MOUNT_POINTS"]:
         log("mounting %s" % (mount_pt))
         # mount_pt must already exist
         if not os.path.exists(mount_pt):
-          raise MaxwellError("Mount point '%s' does not exist", mount_pt)
+          err.write("Mount point '%s' does not exist", mount_pt)
+          continue
         source_mount = mount_pt + user
         # check if source exists
         if not os.path.exists(source_mount):
@@ -341,7 +362,8 @@ class Container():
           p = subprocess.Popen(args, stderr = err, stdout = err)
           p.communicate()
           if p.returncode != 0:
-            raise MaxwellError("Could not create '%s'" % (source_mount))
+            err.write("Could not create '%s'" % (source_mount))
+            continue
         if not os.path.exists(self.vz_root_path + source_mount):
           args = ['/bin/mkdir', '-m', '0700', '-p', self.vz_root_path + source_mount]
           p = subprocess.Popen(args, stderr = err, stdout = err)
@@ -350,12 +372,13 @@ class Container():
             raise MaxwellError("Could not create '%s'" % (self.vz_root_path + source_mount))
         self.__root_mount(source_mount, 'rw,noatime')
 
+    # Mount projects based on user membership
     if self.k["PROJECT_MOUNT"]:
       for g in groups:
-	if g[0:3] == "pr-":
-	  source_mount = self.k["PROJECT_PATH"] + g[3:]
+        if g[0:3] == "pr-":
+          source_mount = self.k["PROJECT_PATH"] + g[3:]
           if not os.path.exists(source_mount):
-	    continue
+            continue
           if not os.path.exists(self.vz_root_path + source_mount):
             args = ['/bin/mkdir', '-m', '0700', '-p', self.vz_root_path + source_mount]
             p = subprocess.Popen(args, stderr = err, stdout = err)
@@ -363,7 +386,81 @@ class Container():
             if p.returncode != 0:
               raise MaxwellError("Could not create '%s'" % (self.vz_root_path + source_mount))
           self.__root_mount(source_mount, 'rw,noatime')
-	  
+
+    # Mount public project file areas
+    # https://mygeohub.org/support/ticket/1226
+    # IMPORTANT:  path for this can be entirely different from path for regular project areas
+    # compare "/srv/mygeohub/projects/" vs "/srv/irods/"
+    # this section uses PROJECT_PUBLIC_PATH instead of PROJECT_PATH
+    # mount read-only for users who do not belong to the corresponding projects
+    # mount read-write for users who belong
+    # public project file areas are:  /srv/mygeohub/projects/*/files/public
+    # algorithm
+    #   for each project group in which the user belongs
+    #     if there's a public area, mount it r/w
+    #   find projects that have "public" directories
+    #     if already mounted, continue
+    #     else mount read-only
+    if self.k["PROJECT_PUBLIC_MOUNT"]:
+      # mount read/write public areas for project members
+      for g in groups:
+        if g[0:3] == "pr-":
+          source_mount = self.k["PROJECT_PUBLIC_PATH"] + g[3:] + "/files/public"
+          if not os.path.exists(source_mount):
+            continue
+          if not os.path.exists(self.vz_root_path + source_mount):
+            args = ['/bin/mkdir', '-m', '0700', '-p', self.vz_root_path + source_mount]
+            p = subprocess.Popen(args, stderr = err, stdout = err)
+            p.communicate()
+            if p.returncode != 0:
+              raise MaxwellError("Could not create '%s'" % (self.vz_root_path + source_mount))
+          self.__root_mount(source_mount, 'rw,noatime')
+      import glob
+      listing = glob.glob(self.k["PROJECT_PUBLIC_PATH"]+ '/*/files/public')
+      for pubpath in listing:
+        mntpt = self.vz_root_path + pubpath
+        if os.path.isdir(mntpt):
+          continue
+        args = ['/bin/mkdir', '-m', '0700', '-p', mntpt]
+        p = subprocess.Popen(args, stderr = err, stdout = err)
+        p.communicate()
+        if p.returncode != 0:
+          raise MaxwellError("Could not create '%s'" % (mntpt))
+        self.__root_mount(pubpath, 'ro,noatime')
+        
+    # Use local storage for the session directory
+    # implemented for cdmhub, to use SSDs on the execution host
+    if self.k["LOCAL_SESSIONDIR"]:
+      import shutil
+      localhome_dir="/home/sessions/" + user
+      local_dir = "/home/sessions/%s/%d" % (user, session_id)
+      remote_dir = account.homedir + "/data/sessions/%d" % session_id
+      # create local user home so operations can be made under the user account and not root
+      args = ['mkdir', '-p', '-m', '0750', localhome_dir]
+      p = subprocess.Popen(args, stderr = err, stdout = err)
+      p.communicate()
+      if p.returncode != 0:
+        raise MaxwellError("Could not create '%s'" % (localhome_dir))
+      os.chown(localhome_dir, account.uid, account.gid)
+
+      # move the session directory to fast local storage
+      args = ['/bin/su', user, '-c', 'mv %s %s' % (remote_dir, localhome_dir)]
+      p = subprocess.Popen(args, stderr = err, stdout = err)
+      p.communicate()
+      if p.returncode != 0:
+        raise MaxwellError("Could not move '%s' to '%s'" % (remote_dir, localhome_dir))
+
+      # mount it
+      # mount: only root can do that
+      # mounting as root will follow symlinks so would be unsafe, can mount over important mount points like /apps
+      # create a symlink from remote_dir to local_dir, as the user
+      # so the openvz mount script has to mount /home/sessions inside the container!
+      args = ['/bin/su', user, '-c', "ln -s %s %s" % (local_dir, remote_dir) ]
+      p = subprocess.Popen(args, stderr = err, stdout = err)
+      p.communicate()
+      if p.returncode != 0:
+        raise MaxwellError("Could not symlink %s to '%s'" % (remote_dir, local_dir))
+
     # Deprecated -- SSHFS-based user mounts -- Deprecated
     if self.k["SSHFS_MOUNT"]:
       # create an SSH connection for each container
@@ -404,7 +501,7 @@ class Container():
       # cmd = "xauth -v source /Xvnc/authlist*"
       # args = ['/usr/sbin/vzctl', 'exec2', str(self.veid), 'su', user, '-s', '/bin/sh', '-c',
       #  '\"cd; %s %s\"' % (env_cmd, cmd)]
-      args = ['/usr/sbin/vzctl', 'exec2', str(self.veid), 'su', user, '-s', '/bin/dash', '-c',
+      args = ['/usr/sbin/vzctl', 'exec2', str(self.veid), 'su', user, '-s', '/bin/sh', '-c',
         self.k["INTERNAL_PATH"] + 'mergeauth']
       if VERBOSE:
         log("command is %s\n" % " ".join(args))
@@ -448,7 +545,7 @@ class Container():
       # Python docs: "The data read is buffered in memory, so do not use this method if the data size is large or unlimited."
       # Problem:  If the tool is misbehaving and has GBs of output, then root processes start
       # consuming GBs of memory!  Todo: find alternative to calling subprocess.communicate while capturing output
-      p = subprocess.Popen(args, stderr = err, stdout = err)
+      p = subprocess.Popen(args, stderr = err, stdout = out)
       p.communicate()
       end_time = time.time()
 
@@ -523,7 +620,11 @@ class Container():
         log("exception:'%s'\n" % exc)
         raise MaxwellError("'%s' already exists and is probably already mounted.  Giving up."
           % mntpt)
-    os.mkdir(mntpt)
+    try:
+      saved_umask = os.umask(0)
+      os.makedirs(mntpt, 0755)
+    finally:
+      os.umask(saved_umask)
     if VERBOSE:
       log("Created %s" % mntpt)
 
@@ -538,6 +639,57 @@ class Container():
         pass
       raise MaxwellError("Could not mount %s in '%d'" % (point, self.veid))
 
+  def __mount_subdirs(self, err, user, topdir):
+    """Bind mount subdirectories of topdir if they are accessible to user.  Determine if they should be read-only or r/w"""
+    if os.path.isdir(topdir):
+      for o in os.listdir(topdir):
+        if o == 'lost+found':
+          continue
+        fp=os.path.join(topdir,o)
+        if os.path.isdir(fp):
+          accessible = False
+          # try to ls the directory, as the user
+          args = ['/bin/su', user, '-c', "ls " + fp]
+          with open(os.devnull, 'w') as devnull:
+            returncode = subprocess.call(['/bin/su', user, '-c', "ls " + fp], stdout=devnull, stderr=devnull)
+          if returncode == 0:
+            accessible = True
+            permissions = 'ro,noatime'
+          # try to touch a file as the user, with "su"
+          with open(os.devnull, 'w') as devnull:
+            returncode = subprocess.call(['/bin/su', user, '-c', "touch %s/.test" % fp], stdout=devnull, stderr=devnull)
+          if returncode == 0:
+            # we can write
+            accessible = True
+            permissions = 'rw,noatime'
+          if accessible:
+            err.write("mounting %s with permissions %s; " %(fp, permissions))
+            self.__root_mount(fp, permissions)
+          else:
+            err.write("cannot mount %s" %(fp))
+
+  def __mount_subdirs_ro(self, err, user, topdir, permissions):
+    """Bind mount subdirectories of topdir with provided permissions if they are readable by the user.
+    The permissions are intended to contain 'ro', as it only tests read access.  Faster than __mount_subdirs"""
+    if os.path.isdir(topdir):
+      for o in os.listdir(topdir):
+        if o == 'lost+found':
+          continue
+        fp=os.path.join(topdir,o)
+        if os.path.isdir(fp):
+          accessible = False
+          # try to ls the directory, as the user
+          args = ['/bin/su', user, '-c', "ls " + fp]
+          with open(os.devnull, 'w') as devnull:
+            returncode = subprocess.call(['/bin/su', user, '-c', "ls " + fp], stdout=devnull, stderr=devnull)
+          if returncode == 0:
+            accessible = True
+          if accessible:
+            err.write("mounting %s with permissions %s; " %(fp, permissions))
+            self.__root_mount(fp, permissions)
+          else:
+            err.write("cannot mount %s" %(fp))
+
   def firewall_by_group(self, groups, operation='add'):
     """groups is an array of groups the user belongs to
     FW_GROUP_MAP is an array of ["group_name", net_cidr, portmin, portmax]
@@ -870,17 +1022,19 @@ class Container():
 
   def setup_template(self):
     """Setup symlinks and mount points for OpenVZ container.
-        usr is problematic, some versions of gcc don't work with a symlink.
-        so even though nanohub worked with a symlink, we're going to try to
-        make usr a mount point instead.
-        The mount point sometimes generates these:
+        usr is a temporary symlink.  It needs to be a symlink initially and later 
+        we replace it with a mount point.  The mount point is needed 
+        because some versions of gcc don't work with a symlink.
+        Doing the mount point without first doing the symlink sometimes generates these:
       # bash: line 318: awk: command not found
       #  ERROR: Can't change file /etc/hosts
     """
-    os.makedirs(self.vz_root_path, mode=0755)
+    os.makedirs(self.vz_root_path)
+    os.chmod(self.vz_root_path, 0755)
     if VERBOSE:
       log("created directory " + self.vz_root_path)
-    os.makedirs(self.vz_private_path, mode=0755)
+    os.makedirs(self.vz_private_path)
+    os.chmod(self.vz_private_path, 0755)
     if VERBOSE:
       log("created directory " + self.vz_private_path)
     # for link in ['bin', 'lib', 'sbin', 'lib64', 'usr']: equivalent to "template copy -a" method
@@ -897,6 +1051,8 @@ class Container():
 
     for vzdir in ['.root', 'home', 'mnt', 'proc', 'sys']:
       os.mkdir(self.vz_private_path + "/" + vzdir)
+      os.chmod(self.vz_private_path + "/" + vzdir, 0755)
+
     if VERBOSE:
       log("template setup")
 
