@@ -257,29 +257,37 @@ class Display:
     the primary key is (host, display) for the display table.
     host:  a Host object"""
 
+    # does this host have any displays already?  If so what's the highest number?
     max_disp = db.getsingle("""
       SELECT MAX(dispnum) FROM display
       WHERE hostname=%s""",
       (host.hostname))
+    first_display = host.get_first_display(db)
     if max_disp is None:
-      dispnum = 1
+      dispnum = first_display
     else:
+      # find an unused display number
+      # is the next highest OK to use?
       dispnum = int(max_disp) + 1
-      if dispnum > 65535: # only two bytes available for IP address (see container.py)
-        min_disp = db.getsingle("""
-          SELECT MIN(dispnum) FROM display
-          WHERE hostname=%s""",
+      # What is the maximum display number? 
+      # < 65536 because only two bytes available for IP address (see container.py)
+      # < maximum number of uses + first_display
+      display_limit = host.max_uses(db) + first_display
+      if dispnum >= display_limit: # 
+        # try to fill gaps because container id range is restricted
+        min_disp = db.getsingle("""SELECT MIN(dispnum) FROM display WHERE hostname=%s""",
           (host.hostname))
-        if int(min_disp) == 1:
+        if min_disp > first_display:
+          dispnum = first_display
+        else:
           min_disp = db.getsingle("""
             SELECT l.dispnum +1 as start FROM display as l
             LEFT OUTER JOIN display as r on l.dispnum +1 = r.dispnum
             WHERE hostname=%s AND r.dispnum IS NULL""",
             (host.hostname))
-          if min_disp is None:
+          dispnum = int(min_disp)
+          if dispnum >= display_limit or dispnum < first_display:
             raise DisplayError("All display numbers used up")
-        else:
-          dispnum = int(min_disp) -1
 
     db.c.execute("""
       INSERT INTO display(hostname,dispnum,status)
@@ -312,32 +320,15 @@ class Display:
         AND display.status='ready'""", \
         (host.hostname))
 
-  @staticmethod
-  def get_ready_disp(c, appinfo, host_k={}):
-    """  DEPRECATED, use get_Display instead
-    find on any host
-    return a ready display appropriate for that application, with the appropriate geometry.
-    Return None if none were found."""
-
-    cmd = """
-      SELECT display.hostname, display.dispnum, display.vncpass
-      FROM display JOIN host ON display.hostname=host.hostname
-      WHERE host.provisions & %d = %d
-      AND host.status='up'
-      AND display.status='ready'""" % (appinfo.hostreq, appinfo.hostreq)
-    row = c.getrow(cmd + """
-      AND display.geometry=%s
-      AND display.depth=%s
-      ORDER BY display.dispnum ASC LIMIT 1"""
-      , (appinfo.geometry, appinfo.depth))
-    if row is None:
-      return None
-    return Display(Host(c, row[0], host_k), row[1], row[2])
-
   @staticmethod
   def get_Display(db, appinfo, sessnum, host_k={}, zone = None, strict_geo = True):
-    """return a 'ready' display to run a tool in the given session
-    Return None if none were found."""
+    """
+    1. Find a host, taking max_uses into account
+    2. on that host, find a 'ready' display
+    3. If found, mark it used, resize display
+       else, call Display.make_Display on the chosen host
+    4. Return None on failure
+    """
     if zone is None:
       h = Host.get_host(db, appinfo.hostreq, host_k)
     else:
@@ -391,24 +382,24 @@ class Display:
     db.lock("display")
     if hostname is None:
       db.c.execute("""
-        UPDATE display SET status = 'purged'
+	UPDATE display SET status = 'purged'
 	WHERE status = 'ready'
         """, ())
     else:
       db.c.execute("""
-        UPDATE display SET status = 'purged'
+	UPDATE display SET status = 'purged'
 	WHERE (status = 'ready' OR (status = 'starting' AND display.sessnum = 0)) AND hostname = %s
         """, (hostname))
     
     db.unlock("display")
     rows = db.getall("""
-      SELECT hostname, dispnum, geometry, depth FROM display
-      WHERE status = 'purged'
+	SELECT hostname, dispnum, geometry, depth FROM display
+	WHERE status = 'purged'
     """, ())
     for row in rows:
       if VERBOSE:
         log("stopping ready display: %s:%d" % (row[0],row[1]))
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       if h.is_windows:
         d.__set_status(db, 'ready', 0) # can't use ready method due to state checks
@@ -438,7 +429,7 @@ class Display:
       # ignore Windows execution hosts
       if row[0].find("winexec") > -1:
         continue
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       d.status = 'stopping'
       vnc_res = h.ask_service("status", [str(row[1])])
@@ -458,7 +449,7 @@ class Display:
       # ignore Windows execution hosts
       if row[0].find("winexec") > -1:
         continue
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       vnc_res = h.ask_service("status", [str(row[1])])
       if vnc_res.find('running') == -1:
@@ -481,7 +472,7 @@ class Display:
       WHERE status = 'purged'
     """, ())
     for row in rows:
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       if h.is_windows:
         d.__set_status(db, 'absent', 0) # can't use ready method due to state checks
@@ -519,7 +510,7 @@ class Display:
     for row in rows:
       if VERBOSE:
         log("stopping ready display: %s:%d" % (row[0],row[1]))
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       if h.is_windows:
         d.__set_status(db, 'ready', 0) # can't use ready method due to state checks
@@ -538,7 +529,7 @@ class Display:
     for row in rows:
       if VERBOSE:
         log("stopping broken display: %s:%d" % (row[0],row[1]))
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       if not h.is_windows:
         try:
@@ -568,7 +559,7 @@ class Display:
     for row in rows:
       if VERBOSE:
         log("stopping zombie display: %s:%d" % (row[0],row[1]))
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       if h.is_windows:
         d.__set_status(db, 'absent', 0) # can't use ready method due to state checks
@@ -587,7 +578,7 @@ class Display:
     for row in rows:
       if VERBOSE:
         log("stopping abandoned display: %s:%d" % (row[0],row[1]))
-      h = Host(db, row[0], {})
+      h = Host(row[0], {})
       d = Display(h, row[1], '')
       if h.is_windows:
         d.__set_status(db, 'absent', 0) # can't use ready method due to state checks
@@ -634,7 +625,7 @@ class Display:
         AND display.status = 'used'""", ())
     if row is None:
       return
-    h = Host(db, row[0], host_k)
+    h = Host(row[0], host_k)
     d = Display(h, row[1], "")
     log("I think display %s on host %s is a zombie, trying to stop it" % (row[1], row[0]))
     d.stop(db)
@@ -652,7 +643,7 @@ class Display:
     if row is None:
       db.unlock("display")
       return
-    h = Host(db, row[0], host_k)
+    h = Host(row[0], host_k)
     d = Display(h, row[1], "")
     db.c.execute("""
       INSERT INTO sessionlog
@@ -673,7 +664,7 @@ class Display:
     )
     if row is None:
       return None
-    h = Host(db, row[0])
+    h = Host(row[0])
     d = Display(h, row[1], "")
     d.sessnum = row[2]
     d.status = row[3]
