@@ -240,10 +240,13 @@ class Host(BasicHost):
 
   """
 
-  def __init__(self, db, hostname, overrides={}):
+  def __init__(self, hostname, overrides={}):
     self.K = HOST_K
     self.K.update(overrides)
     BasicHost.__init__(self, hostname, self.K["SVC_HOST_USER"], self.K["KEY_PATH"])
+    """ decouple database ops from filesystem ops:
+       don't require a database connection just to setup the host
+       make separate class for windows or set is_windows afterwards
     if HOST_K["WINDOWS_SUPPORT"]:
       row = self.is_a(db, 'windowshost')
       self.is_windows = (row is not None)
@@ -254,7 +257,10 @@ class Host(BasicHost):
     else:
       self.is_windows = False
       self.service_path = self.K["SERVICE_PATH"]
-
+    """
+    self.is_windows = False
+    self.service_path = self.K["SERVICE_PATH"]
+   
   def get_status(self, db):
     return db.getsingle("""
       SELECT status FROM host
@@ -288,6 +294,24 @@ class Host(BasicHost):
       WHERE hostname=%s""",
       (self.hostname))
 
+  def max_uses(self, db):
+    """Return max_uses for this host"""
+    return db.getsingle("""
+      SELECT if(max_uses>0, max_uses, 999) FROM host
+      WHERE hostname=%s""",
+      (self.hostname))
+
+  def get_first_display(self, db):
+    """Return the minimum display number to use for this host, or 1 if not set or set to 0"""
+    first_display = db.getsingle("""SELECT first_display FROM host WHERE hostname=%s""",
+      (self.hostname))
+    if first_display is None:
+      return 1
+    v = int(first_display)
+    if v == 0:
+      return 1
+    return v
+
 
   @staticmethod
   def get_host(db, prov_req, host_k = {}, zone = None):
@@ -297,8 +321,8 @@ class Host(BasicHost):
     # Find the least loaded eligible host.
     # note #uses != load
     # note that library can't be used to escape prov_req because it will add quotes!
-    max = db.getsingle("select MAX(max_uses) as max from host ", ())
-    if max is None or max == 0:
+    max_uses = db.getsingle("select MAX(max_uses) as max from host ", ())
+    if max_uses is None or max_uses == 0:
       order_by = 'ORDER BY uses ASC'
     else:
       order_by = 'ORDER BY uses/max_uses ASC'
@@ -324,13 +348,16 @@ class Host(BasicHost):
 
     if row is None:
       return None
-    uses = row[2]
-    max_uses = row[1]
+    uses = int(row[2])
+    if row[1] is None:
+      max_uses = 0
+    else:
+      max_uses = int(row[1])
     if max_uses == 0:
-      return Host(db, row[0], host_k)
+      return Host(row[0], host_k)
     if uses >= max_uses:
       return None
-    return Host(db, row[0], host_k)
+    return Host(row[0], host_k)
 
 
   def create_notify_keys(self, notify_key, quiet):
