@@ -48,6 +48,8 @@ class Session:
   as well as system rules."""
 
   def __init__(self, sessnum, session_suffix="", overrides={}):
+    # foreign: pairs of (organization, username)
+    self.foreign = None
     self.session_suffix = session_suffix
     self.sessnum = int(sessnum)
     self.disp = None
@@ -622,10 +624,10 @@ class Session:
     comm = self.app.command
     if comm.split() == []:
       raise SessionError("Command is empty")
-
-
+      
     # there used to be also a "stream" mode that wrote log files locally
-    # startxapp <user> <sessionid> <timeout> <dispnum> <command>
+    # now we use "notify" to trigger the recovery of log files
+    # API: startxapp <user> <sessionid> <timeout> <dispnum> <command>
     args = [self.user, self.sessname(), str(timeout), str(self.disp.dispnum), "notify"]
 
     if params is not None and params != "":
@@ -642,7 +644,7 @@ class Session:
     count = 0
     while status != 0 and count < 5:
       count += 1
-      # service(self, service_cmd, args, feed="")
+      # API:  service(self, service_cmd, args, feed="")
       try:
         status = self.host.service("startxapp", args)
       except StandardError:
@@ -650,7 +652,35 @@ class Session:
     if count >= 5:
       raise SessionError("unable to invoke command for session %s (app = %s, user= %s)",
                          self.sessnum, self.app.appname, self.user)
-
+  def get_foreign(self, u_account, db):
+    """ mygeohub hydroshare attempt #2
+    From the user_id, get foreign account information
+    should be list of pairs (organization = authenticator, foreign username)
+    if a mount is defined for that organization, mount their resources
+    """
+    self.foreign = db.getall("""SELECT authenticator, username 
+      FROM jos_auth_domain d JOIN jos_auth_link l ON l.auth_domain_id = d.id 
+      WHERE user_id=%s""", (u_account.uid) )
+    #if self.foreign:
+    #  print "foreign mount info is %s" % self.foreign
+      
+  def mount_foreign(self):
+    """ mygeohub hydroshare attempt #2
+    Attempt the mount operations after the database has been closed, 
+    from the results array saved in self.foreign
+    Proof of concept: hydroshare is hard-coded.  In the future,
+    the configuration will have a mapping of mount points to authenticators
+    and we'll check that
+    """
+    if self.foreign is not None:
+      for row in self.foreign:
+        authenticator = row[0]
+        username = row[1]
+        log("authenticator: %s  username: %s" % (authenticator, username))
+        if authenticator == 'hydroshare':
+          #log("mounting %s" % row)
+          self.host.service("mount_paths", [ str(self.disp.dispnum), "/srv/irods/external/hydroshare/%s" % username ])
+          
   def sessname(self):
     return "%d%s" % (self.sessnum, self.session_suffix)
 
@@ -660,17 +690,18 @@ class Session:
         ex = db.getsingle("""SELECT exechost FROM sessionlog WHERE sessnum=%s""", self.sessnum)
         if ex is None or ex == "":
           raise SessionError("local exechost not present in sessionlog '%d', zone %s" % (self.sessnum, self.get_zone(db).zone))
-        self.host = Host(db, ex)
+        self.host = Host(ex)
       else:
         log("zone %s is not local, getting master host name" % self.get_zone(db).zone)
         # zone has a basichost, we're getting a full host here.
-        self.host = Host(db, self.zone.get_host(db).hostname, {"KEY_PATH": self.zone.get_host(db).key_path})
+        self.host = Host(self.zone.get_host(db).hostname, {"KEY_PATH": self.zone.get_host(db).key_path})
     else:
       ex = db.getsingle("""SELECT exechost FROM sessionlog WHERE sessnum=%s""", self.sessnum)
       if ex is None:
+        log("""no results from query SELECT exechost FROM sessionlog WHERE sessnum=%s""", self.sessnum)
         # return None to support jersey_start procedure, instead of raising an exception
         return None
-      self.host = Host(db, ex)
+      self.host = Host(ex)
 
   def get_sessionhost(self, db):
     """Find the execution host or master host name for a session"""
@@ -678,7 +709,7 @@ class Session:
     if ex is None or ex == "":
       self.get_exechost(db)
       return
-    self.host = Host(db, ex)
+    self.host = Host(ex)
 
   def get_stats(self, db, get_stats_retries, get_stats_delay):
     """Get session stats (.err and .out files) after asynchronous notifier."""
@@ -690,6 +721,9 @@ class Session:
     #
     import os
     self.get_exechost(db)
+    if self.host is None:
+      log("execution host not found for session %d, not retrieving logs" % self.sessnum)
+      return
     if VERBOSE:
       log("key path is '%s' for host '%s'" % (self.host.key_path,self.host.hostname))
 
@@ -909,6 +943,9 @@ class Session:
     #
     row = db.getrow("""SELECT walltime,status FROM sessionlog
                       WHERE sessnum=%s""", self.sessnum)
+    if row is None:
+      log("Table sessionlog has no entry for session %d!" % self.sessnum)
+      return      
     rtime = float(row[0])
     status = int(row[1])
 
